using System;
using System.Collections;
using System.Collections.Generic;
using Wisej.Web;

namespace WebApplication2.FileManagement
{
    public enum ListViewColumnType
    {
        //
        // Summary:
        //     Text
        Text = 0,
        //
        // Summary:
        //     Number
        Number = 1,
        //
        // Summary:
        //     Date
        Date = 2,
        //
        // Summary:
        //     Icon
        Icon = 3,
        //
        // Summary:
        //     Control
        Control = 4
    }
    /// <summary>
    /// This class is an implementation of the 'IComparer' interface.
    /// </summary>
    public class FileManagementlvSorter: IComparer<ListViewItem>
    {

        public enum SortType : int
        {
            Text,
            Number,
            DateTime
        }

        private SortType _mTypeToSort = SortType.Text; // default= text

        public SortType TypeToSort
        {
            get { return _mTypeToSort; }
            set { _mTypeToSort = value; }
        }
        /// <summary>
        /// Specifies the column to be sorted
        /// </summary>
        private int _columnToSort;
        /// <summary>
        /// Specifies the order in which to sort (i.e. 'Ascending').
        /// </summary>
        private SortOrder _orderOfSort;
        /// <summary>
        /// Case insensitive comparer object
        /// </summary>
        private CaseInsensitiveComparer _objectCompare;

        private void Init()
        {
            _columnToSort = 0;
            _orderOfSort = SortOrder.None;
            _objectCompare = new CaseInsensitiveComparer();
        }
        public FileManagementlvSorter(ListView inListView)
        {
            // bind the ListView to the Sorter, and set up the ColumnClickEvent
            this.OListView = inListView;
            //this.oListView.ItemSorter = inListView;
            this.OListView.ColumnClick += ColumnHeaderClick;
            Init();
        }

        ~FileManagementlvSorter()
        {
            if (OListView != null)
            {
                OListView.ItemSorter = null;
                this.OListView.ColumnClick -= ColumnHeaderClick;
            }

            OListView = null;
        }
        
        /// <summary>
        /// Class constructor.  Initializes various elements
        /// </summary>
        /// public AFE_lvSorter()
        
        public FileManagementlvSorter()
        {
            Init();
        }

        /// <summary>
        /// This method is inherited from the IComparer interface.  It compares the two objects passed using a case insensitive comparison.
        /// </summary>
        /// <param name="x">First object to be compared</param>
        /// <param name="y">Second object to be compared</param>
        /// <returns>The result of the comparison. "0" if equal, negative if 'x' is less than 'y' and positive if 'x' is greater than 'y'</returns>
        public int Compare(ListViewItem x, ListViewItem y)
        {
            int compareResult = 0;

            // Cast the objects to be compared to ListViewItem objects
            var listviewX = x;
            var listviewY = y;


            if (_columnToSort > listviewX.SubItems.Count-1)
            {
                _columnToSort = listviewX.SubItems.Count - 1;
            }
            // Compare the two items
            if (_mTypeToSort == SortType.Text)
            {
                compareResult = _objectCompare.Compare(listviewX.SubItems[_columnToSort].Text, listviewY.SubItems[_columnToSort].Text);
            }
            else if (_mTypeToSort == SortType.Number)
            {
                try
                {

                    Decimal xValue = FileManagementUtil.SafeDecimalParseWithDefault(listviewX.SubItems[_columnToSort].Text, decimal.MinValue);
                    Decimal yValue = FileManagementUtil.SafeDecimalParseWithDefault(listviewY.SubItems[_columnToSort].Text, decimal.MinValue);

                    if (xValue == decimal.MinValue && yValue == decimal.MinValue)
                    {
                        compareResult = string.Compare(listviewX.SubItems[_columnToSort].Text, listviewY.SubItems[_columnToSort].Text);
                    }
                    else if (xValue  > yValue)
                    {
                        compareResult = 1;
                    }
                    else if (xValue < yValue)
                    {
                        compareResult = -1;
                    }
                    else
                    {
                        compareResult = 0;
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            }
            else if (_mTypeToSort == SortType.DateTime)
            {
                try
                {
                    if (listviewX.SubItems[_columnToSort].Text != "")
                    {
                        DateTime dtX = FileManagementUtil.SafeDateParseWithDefault(listviewX.SubItems[_columnToSort].Text, FileManagementUtil.Jan1900);
                        DateTime dtY = FileManagementUtil.SafeDateParseWithDefault(listviewY.SubItems[_columnToSort].Text, FileManagementUtil.Jan1900);
                        if (dtX > dtY)
                        {
                            compareResult = 1;
                        }
                        else if (dtX < dtY)
                        {
                            compareResult = -1;
                        }
                        else
                        {
                            compareResult = 0;
                        }
                    }
                    else
                    {
                        compareResult = 0;
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                }
            }

           

            // Calculate correct return value based on object comparison
            if (_orderOfSort == SortOrder.Ascending)
            {
                // Ascending sort is selected, return normal result of compare operation
                return compareResult;
            }
            else if (_orderOfSort == SortOrder.Descending)
            {
                // Descending sort is selected, return negative result of compare operation
                return (-compareResult);
            }
            else
            {
                // Return '0' to indicate they are equal
                return 0;
            }
        }

        /// <summary>
        /// Gets or sets the number of the column to which to apply the sorting operation (Defaults to '0').
        /// </summary>
        public int SortColumn
        {
            set
            {
                _columnToSort = value;
            }
            get
            {
                return _columnToSort;
            }
        }

        /// <summary>
        /// Gets or sets the order of sorting to apply (for example, 'Ascending' or 'Descending').
        /// </summary>
        public SortOrder Order
        {
            set
            {
                _orderOfSort = value;
            }
            get
            {
                return _orderOfSort;
            }
        }

        private ListView _oListView = null;
        public ListView OListView
        {
            get { return _oListView; }
            set { _oListView = value; }
        }


        public void ColumnHeaderClick(object sender, ColumnClickEventArgs e)
        {

            // Note: if the sender is a listview, then we can use it as the 
            // column source.
            if (sender is ListView)
            {
                OListView = (ListView)sender;
            }

            if (OListView == null)
            {
                MessageBox.Show("Cannot sort column.. No Listview was available.");
                return;
            }

            ColumnHeader oColHeader = OListView.Columns[e.Column];

            if (OListView.Columns[e.Column].Tag == (object)ListViewColumnType.Text) this.TypeToSort = SortType.Text;
            else if (OListView.Columns[e.Column].Tag == (object)ListViewColumnType.Number) this.TypeToSort = FileManagementlvSorter.SortType.Number;
            else if (OListView.Columns[e.Column].Tag == (object)ListViewColumnType.Date) this.TypeToSort = FileManagementlvSorter.SortType.DateTime;
        

            if (e.Column == this.SortColumn)
            {
                // Reverse the current sort direction for this column.
                if (this.Order == SortOrder.Ascending)
                {
                    Order = SortOrder.Descending;
                }
                else
                {
                    this.Order = SortOrder.Ascending;
                }
            }
            else
            {
                // Set the column number that is to be sorted; default to ascending.
                this.SortColumn = e.Column;
                this.Order = SortOrder.Ascending;
            }

            // Perform the sort with these new sort options.
            OListView.Sort();

        }
    }

}
